=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2011-12-19 13:23:58 +0000
+++ bzrlib/revisiontree.py	2011-12-29 23:15:08 +0000
@@ -103,7 +103,8 @@
         self._inventory = inv
 
     def get_file_mtime(self, file_id, path=None):
-        ie = self._inventory[file_id]
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        ie = inv[inv_file_id]
         try:
             revision = self._repository.get_revision(ie.revision)
         except errors.NoSuchRevision:
@@ -111,26 +112,30 @@
         return revision.timestamp
 
     def get_file_size(self, file_id):
-        return self._inventory[file_id].text_size
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        return inv[inv_file_id].text_size
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):
-        ie = self._inventory[file_id]
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        ie = inv[inv_file_id]
         if ie.kind == "file":
             return ie.text_sha1
         return None
 
     def get_file_revision(self, file_id, path=None):
-        ie = self._inventory[file_id]
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        ie = inv[inv_file_id]
         return ie.revision
 
     def is_executable(self, file_id, path=None):
-        ie = self._inventory[file_id]
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        ie = inv[inv_file_id]
         if ie.kind != "file":
             return False
         return ie.executable
 
     def has_filename(self, filename):
-        return bool(self.inventory.path2id(filename))
+        return bool(self.path2id(filename))
 
     def list_files(self, include_root=False, from_dir=None, recursive=True):
         # The only files returned by this are those from the version
@@ -150,26 +155,30 @@
             yield path, 'V', entry.kind, entry.file_id, entry
 
     def get_symlink_target(self, file_id, path=None):
-        ie = self._inventory[file_id]
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        ie = inv[inv_file_id]
         # Inventories store symlink targets in unicode
         return ie.symlink_target
 
     def get_reference_revision(self, file_id, path=None):
-        return self.inventory[file_id].reference_revision
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        return inv[inv_file_id].reference_revision
 
     def get_root_id(self):
         if self.inventory.root:
             return self.inventory.root.file_id
 
     def kind(self, file_id):
-        return self._inventory[file_id].kind
+        inv, inv_file_id = self._unpack_file_id(file_id)
+        return inv[inv_file_id].kind
 
     def path_content_summary(self, path):
         """See Tree.path_content_summary."""
-        id = self.inventory.path2id(path)
+        id = self.path2id(path)
         if id is None:
             return ('missing', None, None, None)
-        entry = self._inventory[id]
+        inv, inv_file_id = self._unpack_file_id(id)
+        entry = inv[inv_file_id]
         kind = entry.kind
         if kind == 'file':
             return (kind, entry.text_size, entry.executable, entry.text_sha1)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2011-12-19 13:23:58 +0000
+++ bzrlib/tree.py	2011-12-29 23:15:08 +0000
@@ -774,30 +774,44 @@
     inventory = property(_get_inventory,
                          doc="Inventory of this Tree")
 
+    def _unpack_file_id(self, file_id):
+        if isinstance(file_id, tuple):
+            if len(file_id) != 0:
+                raise ValueError("nested trees not yet supported: %r" % file_id)
+            file_id = file_id[0]
+        return self.inventory, file_id
+
     @needs_read_lock
     def path2id(self, path):
         """Return the id for path in this tree."""
-        return self._inventory.path2id(path)
+        return self._path2inv_file_id(path)[1]
+
+    def _path2inv_file_id(self, path):
+        # for now
+        return self.inventory, self._inventory.path2id(path)
 
     def id2path(self, file_id):
         """Return the path for a file id.
 
         :raises NoSuchId:
         """
-        return self.inventory.id2path(file_id)
+        inventory, file_id = self._unpack_file_id(file_id)
+        return inventory.id2path(file_id)
 
     def has_id(self, file_id):
-        return self.inventory.has_id(file_id)
+        inventory, file_id = self._unpack_file_id(file_id)
+        return inventory.has_id(file_id)
 
     def has_or_had_id(self, file_id):
-        return self.inventory.has_id(file_id)
+        inventory, file_id = self._unpack_file_id(file_id)
+        return inventory.has_id(file_id)
 
     def all_file_ids(self):
-        return set(self.inventory)
+        return set(self.inventory) # for now
 
     @deprecated_method(deprecated_in((2, 4, 0)))
     def __iter__(self):
-        return iter(self.inventory)
+        return iter(self.inventory) # for now
 
     def filter_unversioned_files(self, paths):
         """Filter out paths that are versioned.
@@ -807,7 +821,7 @@
         # NB: we specifically *don't* call self.has_filename, because for
         # WorkingTrees that can indicate files that exist on disk but that
         # are not versioned.
-        pred = self.inventory.has_filename
+        pred = self.inventory.has_filename # for now
         return set((p for p in paths if not pred(p)))
 
     @needs_read_lock
@@ -823,8 +837,17 @@
             down to specific_file_ids that have been requested. This has no
             impact if specific_file_ids is None.
         """
+        if specific_file_ids is None:
+            inventory_file_ids = None
+        else:
+            inventory_file_ids = []
+            for tree_file_id in specific_file_ids:
+                inventory, inv_file_id = self._unpack_file_id(tree_file_id)
+                if not inventory is self.inventory: # for now
+                    raise AssertionError("%r != %r" % (inventory, self.inventory))
+                inventory_file_ids.append(inv_file_id)
         return self.inventory.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+            specific_file_ids=inventory_file_ids, yield_parents=yield_parents)
 
     @deprecated_method(deprecated_in((2, 5, 0)))
     def get_file_by_path(self, path):

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-12-19 19:15:58 +0000
+++ bzrlib/workingtree.py	2011-12-29 23:15:08 +0000
@@ -2187,12 +2187,12 @@
             raise
 
     def _is_executable_from_path_and_stat_from_basis(self, path, stat_result):
-        file_id = self.path2id(path)
+        inv, file_id = self._path2inv_file_id(path)
         if file_id is None:
             # For unversioned files on win32, we just assume they are not
             # executable
             return False
-        return self._inventory[file_id].executable
+        return inv[file_id].executable
 
     def _is_executable_from_path_and_stat_from_stat(self, path, stat_result):
         mode = stat_result.st_mode
@@ -2200,7 +2200,8 @@
 
     def is_executable(self, file_id, path=None):
         if not self._supports_executable():
-            return self._inventory[file_id].executable
+            inv, inv_file_id = self._unpack_file_id(file_id)
+            return inv[inv_file_id].executable
         else:
             if not path:
                 path = self.id2path(file_id)
@@ -2452,9 +2453,8 @@
         if not self.is_locked():
             raise errors.ObjectNotLocked(self)
 
-        inv = self.inventory
         if from_dir is None and include_root is True:
-            yield ('', 'V', 'directory', inv.root.file_id, inv.root)
+            yield ('', 'V', 'directory', self.inventory.root.file_id, self.inventory.root)
         # Convert these into local objects to save lookup times
         pathjoin = osutils.pathjoin
         file_kind = self._kind
@@ -2467,12 +2467,13 @@
 
         # directory file_id, relative path, absolute path, reverse sorted children
         if from_dir is not None:
-            from_dir_id = inv.path2id(from_dir)
+            inv, from_dir_id = self._path2inv_file_id(from_dir)
             if from_dir_id is None:
                 # Directory not versioned
                 return
             from_dir_abspath = pathjoin(self.basedir, from_dir)
         else:
+            inv = self.inventory
             from_dir_id = inv.root.file_id
             from_dir_abspath = self.basedir
         children = os.listdir(from_dir_abspath)

